@@ -12,7 +12,7 @@ enum string_param{ STRING_ALPHABET,
 		   STRING_DASHERFONT, 
 		   STRING_USERDIR,
 		   STRING_SYSTEMDIR,
-		   STRING_COLOUR};
+		   STRING_COLOUR };
 
 enum double_param{ DOUBLE_MAXBITRATE };
 
@@ -26,7 +26,15 @@ enum int_param{ INT_LANGUAGEMODEL,
 		INT_ORIENTATION,
 		INT_ENCODING,
 		INT_UNIFORM,
-		INT_ONEBUTTON};
+		INT_ONEBUTTON,
+		INT_LM_MAXORDER,
+		INT_LM_ALPHA,
+		INT_LM_BETA,
+		INT_LM_EXCLUSION,
+		INT_LM_UPDATE_EXCLUSION,
+		INT_LM_MIXTURE,
+		INT_TRUNCATION,
+		INT_TRUNCATIONTYPE };
 
 enum bool_param{ BOOL_DIMENSIONS, 
 		 BOOL_EYETRACKER,
@@ -70,6 +78,8 @@ void dasher_set_parameter_bool( bool_param p, bool value );
 void dasher_set_orientation( Opts::ScreenOrientations orient );
 void dasher_set_encoding( Opts::FileEncodingFormats encoding );
 
+//void dasher_set_lm_parameter( const char *pname, int value );
+
 void dasher_set_string_callback( void(*_cb)( string_param, const char * ) );
 void dasher_set_double_callback( void(*_cb)( double_param, double ) );
 void dasher_set_int_callback( void(*_cb)( int_param, long int ) );
@@ -81,9 +91,11 @@ void dasher_set_colour_scheme_callback( void (*_cb)(int, int*, int*, int*) );
 void dasher_set_draw_rectangle_callback( void (*_cb)(int, int, int, int, int, Opts::ColorSchemes) );
 void dasher_set_draw_polyline_callback( void (*_cb)(Dasher::CDasherScreen::point*, int) );
 void dasher_set_draw_colour_polyline_callback( void (*_cb)(Dasher::CDasherScreen::point*, int, int) );
+void dasher_set_draw_colour_polygon_callback( void (*_cb)(Dasher::CDasherScreen::point*, int, int) );
 void dasher_set_draw_text_callback(void (*_cb)(symbol, int, int, int));
 void dasher_set_draw_text_string_callback(void (*_cb)(std::string, int, int, int));
-void dasher_set_text_size_callback(void (*_cb)(symbol, int*, int*, int));
+void dasher_set_text_size_callback(void (*_cb)(const std::string &, int*, int*, int));
+void dasher_set_send_marker_callback( void (*_cb)( int ) );
 
 void dasher_set_edit_output_callback(void (*_cb)(symbol));
 void dasher_set_edit_outputcontrol_callback(void (*_cb)(void*, int));
@@ -99,6 +111,8 @@ void dasher_set_set_bool_option_callback( void (*_cb)(const std::string&, bool)
 void dasher_set_set_long_option_callback( void (*_cb)(const std::string&, long) );
 void dasher_set_set_string_option_callback( void (*_cb)(const std::string&, const std::string&) );
 
+void dasher_set_input( CDasherInput *_pInput );
+
 void dasher_train_file( const char *filename );
 int dasher_get_alphabets( const char **alphabetlist, int s );
 const char* dasher_get_current_alphabet();
@@ -110,7 +124,7 @@ void dasher_start();
 void dasher_redraw();
 void dasher_render();
 void dasher_tap_on( int x, int y, unsigned long int time );
-void dasher_draw_mouse_position( int x, int y );
+void dasher_draw_mouse_position( int x, int y,int iWhichBox );
 void dasher_go_to( int x, int y );
 void dasher_draw_go_to( int x, int y );
 void dasher_unpause( unsigned long int time );
