@@ -2,6 +2,22 @@
 #include <gdk/gdkx.h>
 #include <glade/glade.h>
 
+#define PREFIX "/usr/"
+#define SYSCONFDIR "/usr/share/dasher/"
+#define LIBDIR "/usr/lib/"
+#define DATADIR "/usr/share/dasher/"
+
+#include <libintl.h>
+#include <locale.h>
+#include <stdio.h>
+#include <stdlib.h>
+//#include <getopt.h>
+
+#include "libdasher.h"
+#include "dasher.h"
+#include "canvas.h"
+#include "edit.h"
+
 #ifndef WITH_GPE
 #include <gconf/gconf.h>
 #include <gconf/gconf-client.h>
@@ -28,22 +44,6 @@ static const struct poptOption options [] =
 };
 #endif
 
-#define PREFIX "/usr/"
-#define SYSCONFDIR "/usr/share/dasher/"
-#define LIBDIR "/usr/lib/"
-#define DATADIR "/usr/share/dasher/"
-
-#include <libintl.h>
-#include <locale.h>
-#include <stdio.h>
-#include <stdlib.h>
-//#include <getopt.h>
-
-#include "libdasher.h"
-#include "dasher.h"
-#include "canvas.h"
-#include "edit.h"
-
 #ifdef WITH_GPE
 #include "gpesettings_store.h"
 #else
@@ -55,6 +55,7 @@ static const struct poptOption options [] =
 #endif
 
 #include "accessibility.h"
+#include "mouse_input.h"
 
 //extern GConfClient *the_gconf_client;
 
@@ -68,6 +69,8 @@ extern int optind;
 extern ControlTree *controltree;
 extern const gchar* filename;
 
+extern int oldx, oldy;
+
 GdkFilterReturn dasher_discard_take_focus_filter (GdkXEvent *xevent, GdkEvent *event, gpointer data)
 {
   XEvent *xev = (XEvent *)xevent;
@@ -99,14 +102,6 @@ main(int argc, char *argv[])
   bind_textdomain_codeset (PACKAGE, "UTF-8");
   textdomain (PACKAGE);
 
-#ifdef GNOME_LIBS
-  GnomeProgram *program=0;
-  program = gnome_program_init("Dasher", PACKAGE_VERSION, LIBGNOMEUI_MODULE, argc, argv, GNOME_PARAM_POPT_TABLE, options, GNOME_PROGRAM_STANDARD_PROPERTIES, GNOME_PARAM_HUMAN_READABLE_NAME, _("Dasher Text Entry"), NULL);
-
-  gnome_vfs_init();
-
-#endif
-
 #ifdef WITH_GPE
   gpe_application_init (&argc, &argv);
   init_xsettings();
@@ -115,6 +110,13 @@ main(int argc, char *argv[])
   gconf_init( argc, argv, &gconferror );
 #endif
 
+#ifdef GNOME_LIBS
+  GnomeProgram *program=0;
+  program = gnome_program_init("Dasher", PACKAGE_VERSION, LIBGNOMEUI_MODULE, argc, argv, GNOME_PARAM_POPT_TABLE, options, GNOME_PROGRAM_STANDARD_PROPERTIES, GNOME_PARAM_HUMAN_READABLE_NAME, _("Dasher Text Entry"), NULL);
+
+  gnome_vfs_init();
+#endif
+
   // We need thread support for updating the splash window while
   // training...
 
@@ -122,12 +124,15 @@ main(int argc, char *argv[])
   g_thread_init(NULL);
 #endif
 
+  
 #ifdef WITH_GPE
   xml = glade_xml_new(PROGDATA"/dashergpe.glade", NULL, NULL);
 #else
   xml = glade_xml_new(PROGDATA"/dasher.glade", NULL, NULL);
   the_gconf_client = gconf_client_get_default();
 #endif
+  if (!xml)
+    return 1;
 
 
 #if (defined GNOME_SPEECH || defined GNOME_A11Y)
@@ -164,6 +169,8 @@ main(int argc, char *argv[])
     }
   }
 
+  
+
   dasher_set_get_bool_option_callback( get_bool_option_callback );
   dasher_set_get_long_option_callback( get_long_option_callback );
   dasher_set_get_string_option_callback( get_string_option_callback );
@@ -183,16 +190,23 @@ main(int argc, char *argv[])
   dasher_set_draw_rectangle_callback( draw_rectangle_callback );
   dasher_set_draw_polyline_callback( draw_polyline_callback );
   dasher_set_draw_colour_polyline_callback( draw_colour_polyline_callback );
+  dasher_set_draw_colour_polygon_callback( draw_colour_polygon_callback );
   dasher_set_draw_text_callback( draw_text_callback );
   dasher_set_draw_text_string_callback( draw_text_string_callback );
   dasher_set_text_size_callback( text_size_callback );
-  
-  dasher_set_edit_output_callback( edit_output_callback );
-  dasher_set_edit_outputcontrol_callback( edit_outputcontrol_callback );
+  dasher_set_send_marker_callback( send_marker_callback );
 
-  dasher_set_edit_delete_callback( edit_delete_callback );
-  dasher_set_get_new_context_callback( get_new_context_callback );
-  dasher_set_clipboard_callback( clipboard_callback );
+
+  dasher_set_edit_output_callback( gtk2_edit_output_callback );
+  dasher_set_edit_outputcontrol_callback( gtk2_edit_outputcontrol_callback );
+
+  dasher_set_edit_delete_callback( gtk2_edit_delete_callback );
+  dasher_set_get_new_context_callback( gtk2_get_new_context_callback );
+  dasher_set_clipboard_callback( gtk2_clipboard_callback );
+
+
+  oldx = -1;
+  oldy = -1;
 
 #ifdef GNOME_A11Y
   SPI_init ();
@@ -219,6 +233,11 @@ main(int argc, char *argv[])
 
   interface_late_setup();
 
+  pMouseInput = new CDasherMouseInput;
+
+  dasher_set_input( pMouseInput );
+
+
 #ifdef WITH_GPE
   gtk_window_set_decorated(GTK_WINDOW(window),false);
 #endif
@@ -250,9 +269,7 @@ main(int argc, char *argv[])
   setup_speech();
 #endif
 
-#ifdef GNOME_A11Y
   setupa11y();
-#endif
 
   controltree=gettree();
   add_control_tree(controltree);
